#!/usr/bin/python2
# -*- coding: utf-8 -*-

import os

from werkzeug.wrappers import Request
from werkzeug.routing import Map, Rule
from werkzeug.wsgi import SharedDataMiddleware
from jinja2 import Environment, FileSystemLoader
from beaker.middleware import SessionMiddleware

from config import Configuration
import database

import views.event
import views.user
import views.location
import views.admin


class EventManagement(object):
    """
    set up eventmanager instance.
    """
    def __init__(self, config_path):
        # set locales to german
        import locale
        locale.setlocale(locale.LC_TIME, 'de_DE.utf-8')

        # load config
        self.config = Configuration(config_path)

        # initialize jinja templates
        self.jinja_env = Environment(
            loader=FileSystemLoader(os.path.join(os.path.dirname(__file__),
                                                 'templates')),
        )

        # rules for URLs
        self.url_map = Map([
            Rule('/', endpoint=views.event.web_list_events),
            Rule('/ical', endpoint=views.event.ical_list_events),
            Rule('/users', endpoint=views.user.web_list_users),
            Rule('/users/<int:user_id>', endpoint=views.user.
                 web_show_user_details),
            Rule('/login', endpoint=views.user.web_login),
            Rule('/logout', endpoint=views.user.web_logout),
            Rule('/admin', endpoint=views.admin.web_admin_welcome),
            Rule('/admin/locations/edit/<int:location_id>',
                 endpoint=views.location.web_edit_location),
            Rule('/admin/locations/delete/<int:location_id>',
                 endpoint=views.location.web_delete_location),
            Rule('/admin/locations/add', endpoint=views.location.
                 web_add_location),
            Rule('/admin/events/delete/<int:event_id>',
                 endpoint=views.event.web_delete_event),
            Rule('/admin/events/add', endpoint=views.event.web_add_event),
            Rule('/admin/events/edit/<int:event_id>',
                 endpoint=views.event.web_edit_event),
            Rule('/admin/users/delete/<int:user_id>',
                 endpoint=views.user.web_delete_user),
            Rule('/admin/users/edit/<int:user_id>',
                 endpoint=views.user.web_edit_user),
            Rule('/admin/users/add', endpoint=views.user.web_add_user)
        ])

    """
    handle incoming WSGI requests.
    """
    def handle_request(self, environment, start_response):
        # wsgi janitoring
        request = Request(environment)
        adapter = self.url_map.bind_to_environ(request.environ)
        endpoint, values = adapter.match()

        # open database connection
        conn = database.connection_factory(self.config)

        # put config and jinja environment in environment variable
        environment['EventManagement.config'] = self.config
        environment['EventManagement.jinja_env'] = self.jinja_env

        # execute views
        response = endpoint(request, environment, conn, **values)

        conn.close()

        return response(environment, start_response)

    """
    redirect incoming WSGI requests to handle_request
    """
    def __call__(self, environment, start_response):
        return self.handle_request(environment, start_response)


def create_app(config_path='blog.cfg', with_static=True):
    app = EventManagement(config_path)

    # adds static directory serving if needed
    #   In production content, leave the web serving
    #   to the _real_ web server instead of the
    #   werkzeug middleware.
    app = SharedDataMiddleware(app, {'/static': 'static', '/assets': 'assets'})

    # adds the beaker Session middleware to store sessions
    app = SessionMiddleware(app)

    return app


def action_initdb():
    """
    Initialize the database and creates admin account.
    """
    conn = database.connection_factory(Configuration('events.cfg'))
    cur = conn.cursor()
    cur.executescript(open('events.sql').read())
    conn.commit()

    from model.user import User
    u = User(username='admin', password='admin', screenname='administrator',
             email='admin@example.com')
    u.db_save(conn)

    conn.close()


def action_adduser(username=('u', ''), password=('p', ''),
                   screenname=('s', ''), email=('e', '')):
    """
    adds a username to the database
    """
    from model.user import User
    conn = database.connection_factory(Configuration('events.cfg'))
    new_user = User(username=username, password=password,
                    screenname=screenname, email=email)
    new_user.db_save(conn)
    print("User %s created!" % username)

if __name__ == '__main__':
    from werkzeug import script

    # actions for argv
    action_runserver = script.make_runserver(create_app, use_debugger=True,
                                             use_reloader=True)
    action_shell = script.make_shell(lambda: {'eventmanagement':
                                              EventManagement(config_path=
                                                              'blog.cfg')})

    script.run()
