import sqlite3
from datetime import datetime, timedelta
from copy import copy

from location import Location
from user import User
import repetitions

class Event(object):
    """
    Represents a real-world event
    """
    id = None
    name = None
    description = None
    location = None
    owner = None
    imagepath = None
    repetition_pattern = None
    begin_date = None
    end_date = None

    def __init__(self, name, begin_date, end_date=None, description=None,
                 location=None, owner=None, imagepath=None):
        self.name = name
        self.begin_date = begin_date
        self.end_date = end_date
        self.description = description
        self.location = location
        self.owner = owner
        self.imagepath = imagepath

    @staticmethod
    def from_dict(d):
        e = Event('', '')
        for key in d.keys():
            e.__dict__[key] = d[key]
        return e

    def db_save(self, conn):
        if self.id is None:
            insert_stmt = "INSERT INTO events (name, description, location_id,\
                    owner_id, imagepath, begin_date, end_date, created_date, \
                    lastchanged_date, repetition_pattern) \
                    VALUES (?,?,?,?,?,?,?,?,?,?)"
            cur = conn.cursor()
            created_date = datetime.now()
            cur.execute(insert_stmt, (self.name, self.description,
                                      self.location.id if not self.location is
                                      None else None,
                                      self.owner.id if not self.owner is None
                                      else None, self.imagepath,
                                      self.begin_date,
                                      self.end_date,
                                      created_date,
                                      created_date,
                                      self.repetition_pattern))
            conn.commit()
            self.created_date = created_date
            self.lastchanged_date = created_date
            self.id = cur.lastrowid
        else:
            update_stmt = "UPDATE events SET name = ?, description = ?, \
                    location_id = ?, owner_id = ?, imagepath = ?, \
                    begin_date = ?, end_date = ?, lastchanged_date = ?, \
                    repetition_pattern = ? WHERE id = ?"

            cur = conn.cursor()
            changetime = datetime.now()
            cur.execute(update_stmt, (self.name, self.description,
                                      self.location.id if not self.location is
                                      None else None,
                                      self.owner.id if not self.owner is None
                                      else None, self.imagepath,
                                      self.begin_date,
                                      self.end_date,
                                      changetime,
                                      self.repetition_pattern,
                                      self.id))
            conn.commit()
            self.lastchanged_date = changetime

    def db_delete(self, conn):
        delete_stmt = "DELETE FROM events WHERE id = ?"
        cur = conn.cursor()
        try:
            cur.execute(delete_stmt, (self.id,))
            conn.commit()
        except sqlite3.IntegrityError:
            cur.close()
            conn.rollback()
            raise Exception('cannot remove user that still has events \
                            associated')

    @staticmethod
    def row_factory(cursor, row):
        state_event = 1
        state_user = 2
        state_location = 3

        dict_event = {}
        dict_location = {}
        dict_user = {}

        current_state = 0

        for idx, col in enumerate(cursor.description):
            if col[0].startswith('START_'):
                current_state += 1
            elif current_state == state_event:
                dict_event[col[0]] = row[idx]
            elif current_state == state_user:
                dict_user[col[0]] = row[idx]
            elif current_state == state_location:
                dict_location[col[0]] = row[idx]

        e = Event.from_dict(dict_event)
        l = Location.from_dict(dict_location)
        u = User.from_dict(dict_user)

        e.owner = u
        e.location = l

        return e

    @classmethod
    def db_load_single(c, conn, event_id):
        select_stmt = "SELECT '' AS 'START_event', t_event.*, '' AS \
                'START_user', t_user.*, '' AS 'START_LOCATION', t_location.* \
                FROM events AS 't_event' INNER JOIN users AS 't_user' ON \
                t_event.owner_id = t_user.id INNER JOIN locations AS \
                t_location ON t_event.location_id = t_location.id WHERE \
                t_event.id = ?;"

        conn.row_factory = c.row_factory
        cur = conn.cursor()
        cur.execute(select_stmt, (event_id,))
        return cur.fetchone()

    @classmethod
    def db_load_multiple(c, conn, offset=None, limit=None, after=None,
                         before=None, future=True):
        select_stmt = "SELECT '' AS 'START_event', t_event.*, '' AS \
                'START_user', t_user.*, '' AS 'START_LOCATION', t_location.* \
                FROM events AS 't_event' INNER JOIN users AS 't_user' ON \
                t_event.owner_id = t_user.id INNER JOIN locations AS \
                t_location ON t_event.location_id = t_location.id "
        if future:
            select_stmt += "WHERE begin_date > DATETIME('now') "

        select_stmt += "ORDER BY begin_date"
        conn.row_factory = c.row_factory
        cur = conn.cursor()
        cur.execute(select_stmt)
        return cur.fetchall()

    @classmethod
    def db_load_all(c, conn):
        return Event.db_load_multiple(conn, future=False)

    @staticmethod
    def resolve_repetions(events, before=None):
        """
        unfolds repeating events in the array and returns the result sorted by
        begin_date
        """
        add_events = []

        for e in events:
            if e.repetition_pattern is None:
                rep = repetitions.repetition_class_by_id[0]()
            else:
                rep = repetitions\
                    .repetition_class_by_id[e.repetition_pattern]()

            add_events.extend(rep.unfold_events(e))

        events.extend(add_events)

        return sorted(events, key=lambda event: event.begin_date)
