import os
from datetime import datetime
from hashlib import md5

from werkzeug.wrappers import Response
from werkzeug.utils import redirect
from wtforms import Form, TextField, FileField, TextAreaField, DateTimeField,\
    SelectField, validators
from PIL import Image
from StringIO import StringIO
import icalendar
from libs.html2text import html2text

from model.user import User
from model.location import Location
from model.event import Event
import model.repetitions
import util
from util import render_template, authenticated


class EventForm(Form):
    name = TextField("Veranstaltungsname", [validators.required()])
    begin_date = TextField("Beginn", [validators.required()])
    end_date = TextField("Ende")
    location = SelectField("Ort", coerce=int)
    repetition = SelectField("Wiederholung", coerce=int,
                 choices=[(r[0], r[1].descriptor) for r in enumerate(model
                          .repetitions.repetition_class_by_id)])
    description = TextAreaField("Beschreibung")
    image = FileField("Bild")


def event_obj_from_form(form, request, conn, event_obj=None):
    """
    Edits or creates an event Object to match the Data passed in form.

    If no event_obj is passed, one will be created and returned.
    """
    if form.validate():
        begin_date = datetime.strptime(form.begin_date.data, '%d.%m.%Y, %H:%M')
        if event_obj is None:
            event_obj = Event(form.name.data, begin_date)
        else:
            event_obj.begin_date = begin_date
            event_obj.name = form.name.data
        if begin_date < datetime.now():
            form.errors['Veranstaltungsbeginn'] = ['Der Veranstaltungsbeginn \
                                                   darf nicht in der \
                                                   Vergangenheit liegen.']
            return None
        if form.end_date.data:
            end_date = datetime.strptime(form.end_date.data, '%d.%m.%Y, %H:%M')
            event_obj.end_date = end_date
            if end_date < begin_date:
                form.errors['Veranstaltungsende'] = ['Das Veranstaltungsende \
                                                     darf nicht vor dem Ver\
                                                     anstaltungs bginn liegen']
                return None
        if form.description.data:
            event_obj.description = form.description.data
        #
        if form.repetition.data == 0:
            event_obj.repetition_pattern = None
        else:
            event_obj.repetition_pattern = form.repetition.data

        # get file and checm mimetype
        file_obj = request.files.get('image')
        if file_obj:
            image_path = util.save_image(file_obj)
            event_obj.imagepath = image_path
        event_obj.location = Location.db_load_single(conn, form.location.data)
        return event_obj
    else:
        return None


def __list_events(conn, offset=None, limit=None, before=None, after=None):
    return Event.resolve_repetions(Event.db_load_multiple(conn,
                                   offset=offset, limit=limit,
                                   before=before, after=after))


def web_list_events(request, environment, conn, page_number=None):
    events = __list_events(conn)

    return render_template("event/web_list_events.html", environment,
                           events=events)


def ical_list_events(request, environment, conn):
    events = __list_events(conn)

    cal = icalendar.Calendar()
    cal.add('prodid', '-//Weltraumpflege.org///NONSGML LabEvents//EN')
    cal.add('version', '2.0')

    for event in events:
        ical_event = icalendar.Event()
        ical_event.add('summary', event.name)
        ical_event.add('dtstart', event.begin_date)
        if event.end_date:
            ical_event.add('dtend', event.end_date)
        if event.description:
            ical_event.add('description', html2text(event.description))
        ical_event.add('dtstamp', event.lastchanged_date)
        ical_event['uid'] = event.created_date.strftime("%s") + str(event.id) \
            + "@events.openlab-augsburg.de"
        cal.add_component(ical_event)

    return Response(cal.to_ical(), mimetype='text/calendar')


@authenticated
def web_add_event(request, environment, conn):
    class CreateEventForm(EventForm):
        pass

    if request.method == 'POST':
        form = CreateEventForm(request.form)
        form.location.choices = [(l.id, l.name) for l in
                                 Location.db_load_all(conn)]
        new_event = event_obj_from_form(form, request, conn)
        if not new_event is None:
            session = environment['beaker.session']
            new_event.owner = User.db_load_single(conn, username=
                                                  session['username'])
            new_event.db_save(conn)

            return redirect("/admin")
        else:
            return render_template("event/web_add_event.html", environment,
                                   form=form)

    else:

        form = CreateEventForm()
        form.location.choices = [(l.id, l.name) for l in
                                 Location.db_load_all(conn)]
        return render_template("event/web_add_event.html", environment,
                               form=form)


@authenticated
def web_edit_event(request, environment, conn, event_id):
    class EditEventForm(EventForm):
        pass
    event_obj = Event.db_load_single(conn, event_id=event_id)
    if request.method != 'POST':
        repetition_id = 0
        if event_obj.repetition_pattern is not None:
            repetition_id = event_obj.repetition_pattern
        form = EditEventForm(location=event_obj.location.id, repetition=
                repetition_id)
        form.location.choices = [(l.id, l.name) for l in
                                 Location.db_load_all(conn)]

        form.name.data = event_obj.name
        form.begin_date.data = event_obj.begin_date.strftime('%d.%m.%Y, %H:%M')
        if event_obj.end_date:
            form.end_date.data = event_obj.end_date.\
                strftime('%d.%m.%Y, %H:%M')
        if event_obj.description:
            form.description.data = event_obj.description

        return render_template("event/web_edit_event.html", environment,
                               form=form)

    else:
        form = EditEventForm(request.form)

        form.location.choices = [(l.id, l.name) for l in
                                 Location.db_load_all(conn)]

        event_obj = event_obj_from_form(form, request, conn,
                                        event_obj=event_obj)
        if event_obj:
            event_obj.db_save(conn)
            return redirect('/admin')
        else:
            return render_template("event/web_edit_event.html", environment,
                                   form=form)


@authenticated
def web_delete_event(request, environoment, conn, event_id):
    event_obj = Event.db_load_single(conn, event_id=event_id)
    event_obj.db_delete(conn)

    return redirect('/admin')
